@@ -1,17 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::SSLStream;
 
 use strict;
 use warnings;
 use base qw( IO::Async::Stream );
-IO::Async::Stream->VERSION( '0.59' );
 
-our $VERSION = '0.12_001';
-$VERSION = eval $VERSION;
+our $VERSION = '0.01';
 
 use IO::Socket::SSL qw( $SSL_ERROR SSL_WANT_READ SSL_WANT_WRITE );
 use POSIX qw( EAGAIN );
@@ -35,51 +33,53 @@ See the main L<IO::Async::SSL> documentation for an example of its use.
 
 =cut
 
-sub _init
+sub _do_ssl_read
 {
    my $self = shift;
-   my ( $params ) = @_;
 
-   $params->{reader} = \&sslread;
-   $params->{writer} = \&sslwrite;
+   $self->SUPER::on_read_ready;
 
-   $self->SUPER::_init( $params );
+   $self->{read_wants_write} = ( $! == EAGAIN && $SSL_ERROR == SSL_WANT_WRITE );
 }
 
-sub sslread
+sub _do_ssl_write
 {
-   my $stream = shift;
-   my ( $fh, undef, $len ) = @_;
+   my $self = shift;
 
-   my $ret = $stream->_sysread( $fh, $_[1], $len );
+   $self->SUPER::on_write_ready;
 
-   my $read_wants_write = !defined $ret && $! == EAGAIN && $SSL_ERROR == SSL_WANT_WRITE;
-   $stream->want_writeready_for_read( $read_wants_write );
+   $self->{write_wants_read} = ( $! == EAGAIN && $SSL_ERROR == SSL_WANT_READ );
+}
 
-   # It's possible SSL_read took all the data out of the filehandle, thus
-   # making it not appear read-ready any more.
-   if( $fh->pending ) {
-      $stream->loop->later( sub { $stream->on_read_ready } );
+sub on_read_ready
+{
+   my $self = shift;
+
+   $self->_do_ssl_read;
+   $self->want_writeready( 1 ) if $self->{read_wants_write};
+
+   $self->_do_ssl_write if $self->{write_wants_read};
+
+   if( $self->read_handle and $self->read_handle->pending ) {
+      $self->get_loop->later( sub { $self->on_read_ready } );
    }
-
-   return $ret;
 }
 
-sub sslwrite
+sub on_write_ready
 {
-   my $stream = shift;
-   my ( $fh, $buf, $len ) = @_;
+   my $self = shift;
 
-   my $ret = $stream->_syswrite( $fh, $_[1], $len );
+   $self->_do_ssl_write;
+   $self->want_readready( 1 ) if $self->{write_wants_read};
 
-   my $write_wants_read = !defined $ret && $! == EAGAIN && $SSL_ERROR == SSL_WANT_READ;
-   $stream->want_readready_for_write( $write_wants_read );
-   # If write wants read, there's no point waiting on writereadiness either
-   $stream->want_writeready_for_write( !$write_wants_read );
-
-   return $ret;
+   $self->_do_ssl_read if $self->{read_wants_write};
 }
 
+# Keep perl happy; keep Britain tidy
+1;
+
+__END__
+
 =head1 BUGS
 
 =over 4
@@ -98,7 +98,3 @@ with regular C<write> with C<autoflush> turned off.
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
